@@ -93,9 +93,12 @@ typedef struct _CoglFramebuffer CoglFramebuffer;
 #include <cogl/cogl-xlib.h>
 #include <cogl/cogl-xlib-renderer.h>
 #endif
-#if COGL_HAS_EGL_PLATFORM_WAYLAND_SUPPORT
+#if defined (COGL_HAS_EGL_PLATFORM_WAYLAND_SUPPORT)
 #include <cogl/cogl-wayland-renderer.h>
 #endif
+#if COGL_HAS_WIN32_SUPPORT
+#include <cogl/cogl-win32-renderer.h>
+#endif
 /* XXX: This will definitly go away once all the Clutter winsys
  * code has been migrated down into Cogl! */
 #include <cogl/cogl-clutter.h>
@@ -206,58 +209,6 @@ cogl_get_bitmasks (int *red,
                    int *blue,
                    int *alpha);
 
-/*
- * _cogl_framebuffer_get_red_bits:
- * @framebuffer: a handle for a framebuffer
- *
- * Retrieves the number of red bits of @framebuffer
- *
- * Return value: the number of bits
- *
- * Since: 1.4
- */
-int
-_cogl_framebuffer_get_red_bits (CoglFramebuffer *framebuffer);
-
-/*
- * _cogl_framebuffer_get_green_bits:
- * @framebuffer: a handle for a framebuffer
- *
- * Retrieves the number of green bits of @framebuffer
- *
- * Return value: the number of bits
- *
- * Since: 1.4
- */
-int
-_cogl_framebuffer_get_green_bits (CoglFramebuffer *framebuffer);
-
-/*
- * _cogl_framebuffer_get_blue_bits:
- * @framebuffer: a handle for a framebuffer
- *
- * Retrieves the number of blue bits of @framebuffer
- *
- * Return value: the number of bits
- *
- * Since: 1.4
- */
-int
-_cogl_framebuffer_get_blue_bits (CoglFramebuffer *framebuffer);
-
-/*
- * _cogl_framebuffer_get_alpha_bits:
- * @framebuffer: a handle for a framebuffer
- *
- * Retrieves the number of alpha bits of @framebuffer
- *
- * Return value: the number of bits
- *
- * Since: 1.4
- */
-int
-_cogl_framebuffer_get_alpha_bits (CoglFramebuffer *framebuffer);
-
 /**
  * cogl_perspective:
  * @fovy: Vertical of view angle in degrees.
