@@ -212,9 +212,19 @@ static const CoglFeatureData winsys_feature_data[] =
   };
 
 static CoglFuncPtr
-_cogl_winsys_get_proc_address (const char *name)
+_cogl_winsys_renderer_get_proc_address (CoglRenderer *renderer,
+                                        const char *name)
 {
-  return (CoglFuncPtr) eglGetProcAddress (name);
+  void *ptr;
+
+  ptr = eglGetProcAddress (name);
+
+  /* eglGetProcAddress doesn't support fetching core API so we need to
+     get that separately with GModule */
+  if (ptr == NULL)
+    g_module_symbol (renderer->libgl_module, name, &ptr);
+
+  return ptr;
 }
 
 #ifdef COGL_HAS_EGL_PLATFORM_ANDROID_SUPPORT
@@ -342,7 +352,6 @@ static void
 check_egl_extensions (CoglRenderer *renderer)
 {
   CoglRendererEGL *egl_renderer = renderer->winsys;
-  const CoglWinsysVtable *winsys = renderer->winsys_vtable;
   const char *egl_extensions;
   int i;
 
@@ -352,7 +361,7 @@ check_egl_extensions (CoglRenderer *renderer)
 
   egl_renderer->private_features = 0;
   for (i = 0; i < G_N_ELEMENTS (winsys_feature_data); i++)
-    if (_cogl_feature_check (winsys,
+    if (_cogl_feature_check (renderer,
                              "EGL", winsys_feature_data + i, 0, 0,
                              COGL_DRIVER_GL, /* the driver isn't used */
                              egl_extensions,
@@ -836,6 +845,40 @@ try_create_context (CoglDisplay *display,
                      EGL_HEIGHT,
                      &egl_display->egl_surface_height);
   }
+
+#elif defined (COGL_HAS_EGL_PLATFORM_GDL_SUPPORT)
+
+  egl_display->egl_surface =
+    eglCreateWindowSurface (edpy,
+                            config,
+                            (NativeWindowType) display->gdl_plane,
+                            NULL);
+
+  if (egl_display->egl_surface == EGL_NO_SURFACE)
+    {
+      error_message = "Unable to create EGL window surface";
+      goto fail;
+    }
+
+  if (!eglMakeCurrent (egl_renderer->edpy,
+                       egl_display->egl_surface,
+                       egl_display->egl_surface,
+                       egl_display->egl_context))
+    {
+      error_message = "Unable to eglMakeCurrent with egl surface";
+      goto fail;
+    }
+
+  eglQuerySurface (egl_renderer->edpy,
+                   egl_display->egl_surface,
+                   EGL_WIDTH,
+                   &egl_display->egl_surface_width);
+
+  eglQuerySurface (egl_renderer->edpy,
+                   egl_display->egl_surface,
+                   EGL_HEIGHT,
+                   &egl_display->egl_surface_height);
+
 #elif defined (COGL_HAS_EGL_PLATFORM_POWERVR_NULL_SUPPORT)
 
   egl_display->egl_surface =
@@ -910,7 +953,14 @@ cleanup_context (CoglDisplay *display)
       egl_display->egl_context = EGL_NO_CONTEXT;
     }
 
-#ifdef COGL_HAS_EGL_PLATFORM_POWERVR_X11_SUPPORT
+#if defined (COGL_HAS_EGL_PLATFORM_POWERVR_NULL_SUPPORT) || \
+    defined (COGL_HAS_EGL_PLATFORM_GDL_SUPPORT)
+  if (egl_display->egl_surface != EGL_NO_SURFACE)
+    {
+      eglDestroySurface (egl_renderer->edpy, egl_display->egl_surface);
+      egl_display->egl_surface = EGL_NO_SURFACE;
+    }
+#elif COGL_HAS_EGL_PLATFORM_POWERVR_X11_SUPPORT
   if (egl_display->dummy_surface != EGL_NO_SURFACE)
     {
       eglDestroySurface (egl_renderer->edpy, egl_display->dummy_surface);
@@ -998,6 +1048,7 @@ gdl_plane_init (CoglDisplay *display, GError **error)
 {
   gboolean ret = TRUE;
   gdl_color_space_t colorSpace = GDL_COLOR_SPACE_RGB;
+  gdl_pixel_format_t pixfmt = GDL_PF_ARGB_32;
   gdl_rectangle_t dstRect;
   gdl_display_info_t display_info;
   gdl_ret_t rc = GDL_SUCCESS;
@@ -1034,9 +1085,9 @@ gdl_plane_init (CoglDisplay *display, GError **error)
   dstRect.height = display_info.tvmode.height;
 
   /* Configure the plane attribute. */
-  rc = gdl_plane_reset (plane);
+  rc = gdl_plane_reset (display->gdl_plane);
   if (rc == GDL_SUCCESS)
-    rc = gdl_plane_config_begin (plane);
+    rc = gdl_plane_config_begin (display->gdl_plane);
 
   if (rc == GDL_SUCCESS)
     rc = gdl_plane_set_attr (GDL_PLANE_SRC_COLOR_SPACE, &colorSpace);
@@ -1047,9 +1098,15 @@ gdl_plane_init (CoglDisplay *display, GError **error)
   if (rc == GDL_SUCCESS)
     rc = gdl_plane_set_attr (GDL_PLANE_DST_RECT, &dstRect);
 
+  /* Default to triple buffering if we don't have an onscreen template */
   if (rc == GDL_SUCCESS)
-    rc = gdl_plane_set_uint (GDL_PLANE_NUM_GFX_SURFACES,
-                             display->swap_chain->length);
+    {
+      if (display->onscreen_template)
+        rc = gdl_plane_set_uint (GDL_PLANE_NUM_GFX_SURFACES,
+                                 display->onscreen_template->swap_chain->length);
+      else
+        rc = gdl_plane_set_uint (GDL_PLANE_NUM_GFX_SURFACES, 3);
+    }
 
   if (rc == GDL_SUCCESS)
     rc = gdl_plane_config_end (GDL_FALSE);
@@ -1064,6 +1121,8 @@ gdl_plane_init (CoglDisplay *display, GError **error)
     }
 
   gdl_close ();
+
+  return TRUE;
 }
 #endif
 
@@ -1317,7 +1376,8 @@ _cogl_winsys_onscreen_init (CoglOnscreen *onscreen,
   wl_surface_map_toplevel (egl_onscreen->wayland_surface);
 
 #elif defined (COGL_HAS_EGL_PLATFORM_POWERVR_NULL_SUPPORT) || \
-      defined (COGL_HAS_EGL_PLATFORM_ANDROID_SUPPORT)
+      defined (COGL_HAS_EGL_PLATFORM_ANDROID_SUPPORT)      || \
+      defined (COGL_HAS_EGL_PLATFORM_GDL_SUPPORT)
   if (egl_display->have_onscreen)
     {
       g_set_error (error, COGL_WINSYS_ERROR,
@@ -1428,7 +1488,8 @@ _cogl_winsys_onscreen_bind (CoglOnscreen *onscreen)
                       egl_display->egl_context);
       egl_context->current_surface = egl_display->dummy_surface;
 #elif defined (COGL_HAS_EGL_PLATFORM_POWERVR_NULL_SUPPORT) || \
-      defined (COGL_HAS_EGL_PLATFORM_ANDROID_SUPPORT)
+      defined (COGL_HAS_EGL_PLATFORM_ANDROID_SUPPORT)      || \
+      defined (COGL_HAS_EGL_PLATFORM_GDL_SUPPORT)
       return;
 #else
 #error "Unknown EGL platform"
@@ -1451,12 +1512,26 @@ _cogl_winsys_onscreen_bind (CoglOnscreen *onscreen)
 
 static void
 _cogl_winsys_onscreen_swap_region (CoglOnscreen *onscreen,
-                                   int *rectangles,
+                                   const int *user_rectangles,
                                    int n_rectangles)
 {
   CoglContext *context = COGL_FRAMEBUFFER (onscreen)->context;
   CoglRendererEGL *egl_renderer = context->display->renderer->winsys;
   CoglOnscreenEGL *egl_onscreen = onscreen->winsys;
+  CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (onscreen);
+  int framebuffer_height  = cogl_framebuffer_get_height (framebuffer);
+  int *rectangles = g_alloca (sizeof (int) * n_rectangles * 4);
+  int i;
+
+  /* eglSwapBuffersRegion expects rectangles relative to the bottom left corner
+   * but we are given rectangles relative to the top left so we need to flip
+   * them... */
+  memcpy (rectangles, user_rectangles, sizeof (int) * n_rectangles * 4);
+  for (i = 0; i < n_rectangles; i++)
+    {
+      int *rect = &rectangles[4 * i];
+      rect[1] = framebuffer_height - rect[1] - rect[3];
+    }
 
   if (egl_renderer->pf_eglSwapBuffersRegion (egl_renderer->edpy,
                                              egl_onscreen->egl_surface,
@@ -1657,7 +1732,7 @@ static CoglWinsysVtable _cogl_winsys_vtable =
   {
     .id = COGL_WINSYS_ID_EGL,
     .name = "EGL",
-    .get_proc_address = _cogl_winsys_get_proc_address,
+    .renderer_get_proc_address = _cogl_winsys_renderer_get_proc_address,
     .renderer_connect = _cogl_winsys_renderer_connect,
     .renderer_disconnect = _cogl_winsys_renderer_disconnect,
     .display_setup = _cogl_winsys_display_setup,
@@ -1800,11 +1875,21 @@ _cogl_egl_create_image (CoglContext *ctx,
 {
   CoglDisplayEGL *egl_display = ctx->display->winsys;
   CoglRendererEGL *egl_renderer = ctx->display->renderer->winsys;
+  EGLContext egl_ctx;
 
   g_return_val_if_fail (egl_renderer->pf_eglCreateImage, EGL_NO_IMAGE_KHR);
 
+  /* The EGL_KHR_image_pixmap spec explicitly states that EGL_NO_CONTEXT must
+   * always be used in conjunction with the EGL_NATIVE_PIXMAP_KHR target */
+#ifdef EGL_KHR_image_pixmap
+  if (target == EGL_NATIVE_PIXMAP_KHR)
+    egl_ctx = EGL_NO_CONTEXT;
+  else
+#endif
+    egl_ctx = egl_display->egl_context;
+
   return egl_renderer->pf_eglCreateImage (egl_renderer->edpy,
-                                          egl_display->egl_context,
+                                          egl_ctx,
                                           target,
                                           buffer,
                                           attribs);
